using UnityEngine;
using System.Collections;
using System.Collections.Generic;
using System.IO;
using System.IO.Compression;
using Newtonsoft.Json;
using Newtonsoft.Json.Linq;
using System.Linq; 
using System; 

namespace SpaceBox
{
    public class SpaceManager : MonoBehaviour
    {
        private static GameObject spaceGameObject;
        private static List<GameObject> existingGameObjects = new List<GameObject>();
        private static bool isSpaceEnabled = false;
        public static SpaceManager instance;

        private void Awake()
        {

                instance = this;
                DontDestroyOnLoad(gameObject); 
        }

        private void Start()
        {
			ListObjectsWithDiscordTracker();
        ModifyDiscordSetting(true);

	}

    public void ModifyDiscordSetting(bool disable)
    {

        if (typeof(Config).GetField("disableDiscord") != null)
        {

            Config.disableDiscord = disable;
            Debug.Log($"Config.disableDiscord has been set to {disable}");
        }
        else
        {
            Debug.LogError("Config.disableDiscord property not found.");
        }
    }

        public static void EnableSpace()
        {
            if (instance == null || isSpaceEnabled) return;
            instance.StartCoroutine(instance.EnableSpaceCoroutine());
        }

        private IEnumerator EnableSpaceCoroutine()
        {

            if (instance == null)
            {
                Debug.LogError("SpaceManager instance is not initialized.");
                yield break;
            }

            ScanExistingGameObjects();

            yield return new WaitForSeconds(2);

            foreach (var obj in existingGameObjects)
            {
                if (obj != instance.gameObject && obj != null) 
                    obj.SetActive(false);
            }

            if (spaceGameObject == null)
            {
                spaceGameObject = new GameObject("SpaceGameObject");

                Camera spaceCamera = spaceGameObject.AddComponent<Camera>();
                spaceCamera.orthographic = true;
                spaceCamera.orthographicSize = 5;
                spaceCamera.clearFlags = CameraClearFlags.SolidColor;
                spaceCamera.backgroundColor = Color.black;

            spaceGameObject.SetActive(true);

                StarManager starManager = spaceGameObject.AddComponent<StarManager>();

                GameObject planetManagerObject = new GameObject("PlanetManager");
                planetManagerObject.AddComponent<PlanetManager>();

            }

            isSpaceEnabled = true;
        }

public static void DisableSpace()
{
    if (instance == null || !isSpaceEnabled)
    {
        Debug.LogWarning("SpaceManager is either not initialized or space is already disabled.");
        return;
    }

    try
    {
        if (spaceGameObject != null)
        {
            Debug.Log($"Destroying spaceGameObject: {spaceGameObject.name}");
            Destroy(spaceGameObject);
            spaceGameObject = null;
        }

        if (existingGameObjects.Count > 0)
        {
            foreach (var obj in existingGameObjects)
            {
                if (obj != instance.gameObject && obj != null) 
                    obj.SetActive(true);
            }
		}

        isSpaceEnabled = false;
    }
    catch (Exception ex)
    {
        Debug.LogError($"Exception in DisableSpace: {ex.Message}");
    }
}

public static void GeneratePlanet(string planetName, string planetType)
{
    Debug.Log($"Generating planet: {planetName}");

    SaveManager SaveManager = FindObjectOfType<SaveManager>();
    if (SaveManager == null)
    {
        GameObject[] allObjects = Resources.FindObjectsOfTypeAll<GameObject>();
        foreach (GameObject obj in allObjects)
        {
            if (!obj.activeInHierarchy)
            {
                SaveManager = obj.GetComponent<SaveManager>();
                if (SaveManager != null)
                {
                    obj.SetActive(true);
                    break;
                }
            }
        }
    }

    if (SaveManager == null)
    {
        Debug.LogError("SaveManager instance not found.");
        return;
    }

	          string planetCountFilePath = Path.Combine(Application.persistentDataPath, "SpaceBox", "PlanetCount.txt");

            int planetCount = 1;
            if (File.Exists(planetCountFilePath))
            {
                string countText = File.ReadAllText(planetCountFilePath);
                if (int.TryParse(countText, out int count))
                {
                    planetCount = count + 1;
                }
            }

            File.WriteAllText(planetCountFilePath, planetCount.ToString());

            Debug.Log($"Planet Count: {planetCount}");

    string currentGalaxyName = "Crabby Way"; 
    string currentPlanetName = PlanetManager.instance.GetCurrentPlanet();

    string spaceBoxPath = Path.Combine(Application.persistentDataPath, "spacebox");
    Debug.Log($"SpaceBox Path: {spaceBoxPath}");
    string[] starJsonFiles = Directory.GetFiles(spaceBoxPath, "star.json", SearchOption.AllDirectories);
    Debug.Log($"Found {starJsonFiles.Length} star.json files");

    string saveDirectory = null;
    foreach (string starFile in starJsonFiles)
    {

        if (IsPlanetInStarFile(starFile, currentPlanetName))
        {
            saveDirectory = Path.GetDirectoryName(starFile);
            Debug.Log($"Save directory found: {saveDirectory}");
            break;
        }
    }

    if (saveDirectory == null)
    {
        Debug.LogError($"Planet {currentPlanetName} not found in any star file.");
        return;
    }

    string newSaveDirectory = Path.Combine(saveDirectory, currentPlanetName); 
    SaveManager.saveWorldToDirectory(newSaveDirectory);

string nextPlanetLoadDirectory = null;
string[] allDirectories = Directory.GetDirectories(spaceBoxPath, "*", SearchOption.AllDirectories);

foreach (string directory in allDirectories)
{
    if (Path.GetFileName(directory).Equals(planetName, StringComparison.OrdinalIgnoreCase))
    {
        nextPlanetLoadDirectory = directory;
        Debug.Log($"Found directory for planet {planetName}: {nextPlanetLoadDirectory}");
        PlanetManager.instance.SetCurrentPlanet(planetName);
        DisableSpace();
        SaveManager.loadWorld(nextPlanetLoadDirectory, false);
		PlanetManager.instance.ShowTouchdownGUI(planetType);
        return;
    }
}

if (nextPlanetLoadDirectory == null)
{
    Debug.LogError($"No subfolder named {planetName} found in any directory.");
}

            PlanetManager.instance.SetCurrentPlanet(planetName);

            DisableSpace();
			PlanetGenerator.ChoosePlanetBiomes(planetType);
            MapBox.instance.generateNewMap();
			PlanetManager.instance.ShowTouchdownGUI(planetType);
}

        public static bool IsPlanetInStarFile(string starFile, string planetName)
        {
            if (!File.Exists(starFile))
            {
                Debug.LogError($"Star file does not exist: {starFile}");
                return false;
            }

            string starData = File.ReadAllText(starFile);

            try
            {

                Star star = JsonConvert.DeserializeObject<Star>(starData, new StarConverter());

                return star.planetInfo.Any(p => p.name.Equals(planetName, StringComparison.OrdinalIgnoreCase));
            }
            catch (JsonException ex)
            {
                Debug.LogError($"Failed to deserialize star file: {starFile}. Exception: {ex.Message}");
                return false;
            }
        }

private static void ScanExistingGameObjects()
{
    existingGameObjects.Clear();
    UnityEngine.GameObject[] allObjects = UnityEngine.Object.FindObjectsOfType<UnityEngine.GameObject>();

    foreach (var obj in allObjects)
    {
        if (obj != spaceGameObject && obj.activeInHierarchy && obj.GetComponent<DiscordTracker>() == null)
        {
            existingGameObjects.Add(obj);
        }
    }
}

public static void ListObjectsWithDiscordTracker()
{

    DiscordTracker[] discordTrackers = UnityEngine.Object.FindObjectsOfType<DiscordTracker>();

    Debug.Log($"Found {discordTrackers.Length} GameObjects with DiscordTracker:");

    foreach (var tracker in discordTrackers)
    {
        Debug.Log(tracker.gameObject.name);
    }
}

        private void Update()
        {
            if (isSpaceEnabled)
            {
                MoveCamera();
            }
        }

        private void MoveCamera()
        {
            float moveSpeed = 5f;
            float h = Input.GetAxis("Horizontal");
            float v = Input.GetAxis("Vertical");

            if (Camera.main != null)
            {
                Camera.main.transform.Translate(new Vector3(h, v, 0) * moveSpeed * Time.deltaTime);
            }
        }
    }
}